@@ -1,21 +1,34 @@
+import os
+import re
 from collections import Counter
 from subprocess import check_output
 
-import os
-
 root_path = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))
 
+aliases = {
+    re.compile("Jun Omae"): "Jun Omae",
+    re.compile(r"^Hugo$"): "Hugo van Kemenade",
+    re.compile(r"^Tomas R([.])?"): "Tomas R.",
+}
+
+
+def map_alias(name):
+    for pattern, alias in aliases.items():
+        if pattern.match(name):
+            return alias
+    return name
+
 
 def get_sorted_authors_list():
     authors = check_output(['git', 'log', '--format=%aN'], cwd=root_path).decode('UTF-8')
-    counts = Counter(authors.splitlines())
+    counts = Counter(map_alias(name) for name in authors.splitlines())
     return [author for (author, count) in counts.most_common()]
 
 
 def get_authors_file_content():
-    author_list = '\n'.join('- %s' % a for a in get_sorted_authors_list())
+    author_list = "\n".join(f"- {a}" for a in get_sorted_authors_list())
 
-    return '''
+    return f'''
 Babel is written and maintained by the Babel team and various contributors:
 
 {author_list}
@@ -27,7 +40,7 @@ following copyright notice holds true for releases before 2013: "Copyright (c)
 In addition to the regular contributions Babel includes a fork of Lennart
 Regebro's tzlocal that originally was licensed under the CC0 license.  The
 original copyright of that project is "Copyright 2013 by Lennart Regebro".
-'''.format(author_list=author_list)
+'''
 
 
 def write_authors_file():
