@@ -1,14 +1,14 @@
 #
-# Copyright (C) 2007-2011 Edgewall Software, 2013-2023 the Babel team
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
 # All rights reserved.
 #
 # This software is licensed as described in the file LICENSE, which
 # you should have received as part of this distribution. The terms
-# are also available at http://babel.edgewall.org/wiki/License.
+# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://babel.edgewall.org/log/.
+# history and logs, available at https://github.com/python-babel/babel/commits/master/.
 
 import copy
 import datetime
@@ -39,6 +39,24 @@ class MessageTestCase(unittest.TestCase):
         assert catalog.PYTHON_FORMAT.search('foo %(name)*.*f')
         assert catalog.PYTHON_FORMAT.search('foo %()s')
 
+    def test_python_brace_format(self):
+        assert not catalog._has_python_brace_format('')
+        assert not catalog._has_python_brace_format('foo')
+        assert not catalog._has_python_brace_format('{')
+        assert not catalog._has_python_brace_format('}')
+        assert not catalog._has_python_brace_format('{} {')
+        assert not catalog._has_python_brace_format('{{}}')
+        assert catalog._has_python_brace_format('{}')
+        assert catalog._has_python_brace_format('foo {name}')
+        assert catalog._has_python_brace_format('foo {name!s}')
+        assert catalog._has_python_brace_format('foo {name!r}')
+        assert catalog._has_python_brace_format('foo {name!a}')
+        assert catalog._has_python_brace_format('foo {name!r:10}')
+        assert catalog._has_python_brace_format('foo {name!r:10.2}')
+        assert catalog._has_python_brace_format('foo {name!r:10.2f}')
+        assert catalog._has_python_brace_format('foo {name!r:10.2f} {name!r:10.2f}')
+        assert catalog._has_python_brace_format('foo {name!r:10.2f=}')
+
     def test_translator_comments(self):
         mess = catalog.Message('foo', user_comments=['Comment About `foo`'])
         assert mess.user_comments == ['Comment About `foo`']
@@ -342,10 +360,19 @@ def test_message_pluralizable():
 
 
 def test_message_python_format():
+    assert not catalog.Message('foo').python_format
+    assert not catalog.Message(('foo', 'foo')).python_format
     assert catalog.Message('foo %(name)s bar').python_format
     assert catalog.Message(('foo %(name)s', 'foo %(name)s')).python_format
 
 
+def test_message_python_brace_format():
+    assert not catalog.Message('foo').python_brace_format
+    assert not catalog.Message(('foo', 'foo')).python_brace_format
+    assert catalog.Message('foo {name} bar').python_brace_format
+    assert catalog.Message(('foo {name}', 'foo {name}')).python_brace_format
+
+
 def test_catalog():
     cat = catalog.Catalog(project='Foobar', version='1.0',
                           copyright_holder='Foo Company')
@@ -414,6 +441,17 @@ def test_catalog_mime_headers_set_locale():
     ]
 
 
+def test_catalog_mime_headers_type_coercion():
+    """
+    Test that mime headers' keys and values are coerced to strings
+    """
+    cat = catalog.Catalog(locale='de_DE', project='Foobar', version='1.0')
+    # This is a strange interface in that it doesn't actually overwrite all
+    # of the MIME headers, but just sets the ones that are passed in (and known).
+    cat.mime_headers = {b'REPORT-MSGID-BUGS-TO': 8}.items()
+    assert dict(cat.mime_headers)['Report-Msgid-Bugs-To'] == '8'
+
+
 def test_catalog_num_plurals():
     assert catalog.Catalog(locale='en').num_plurals == 2
     assert catalog.Catalog(locale='ga').num_plurals == 5
