@@ -1,14 +1,14 @@
 #
-# Copyright (C) 2007-2011 Edgewall Software, 2013-2023 the Babel team
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
 # All rights reserved.
 #
 # This software is licensed as described in the file LICENSE, which
 # you should have received as part of this distribution. The terms
-# are also available at http://babel.edgewall.org/wiki/License.
+# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://babel.edgewall.org/log/.
+# history and logs, available at https://github.com/python-babel/babel/commits/master/.
 
 import calendar
 from datetime import date, datetime, time, timedelta
@@ -17,7 +17,7 @@ import freezegun
 import pytest
 
 from babel import Locale, dates
-from babel.dates import NO_INHERITANCE_MARKER, UTC, _localize
+from babel.dates import NO_INHERITANCE_MARKER, UTC, _localize, parse_pattern
 from babel.util import FixedOffsetTimezone
 
 
@@ -619,6 +619,19 @@ def test_format_skeleton(timezone_getter):
     assert (dates.format_skeleton('EHm', dt, tzinfo=timezone_getter('Asia/Bangkok'), locale='th') == 'อา. 22:30 น.')
 
 
+@pytest.mark.parametrize(('skeleton', 'expected'), [
+    ('Hmz', 'Hmv'),
+    ('kmv', 'Hmv'),
+    ('Kmv', 'hmv'),
+    ('Hma', 'Hm'),
+    ('Hmb', 'Hm'),
+    ('zkKab', 'vHh'),
+])
+def test_match_skeleton_alternate_characters(skeleton, expected):
+    # https://github.com/unicode-org/icu/blob/5e22f0076ec9b55056cd8a84e9ef370632f44174/icu4j/main/core/src/main/java/com/ibm/icu/text/DateIntervalInfo.java#L1090-L1102
+    assert dates.match_skeleton(skeleton, (expected,)) == expected
+
+
 def test_format_timedelta():
     assert (dates.format_timedelta(timedelta(weeks=12), locale='en_US')
             == '3 months')
@@ -643,6 +656,13 @@ def test_parse_date():
     assert dates.parse_date('2004-04-01', locale='sv_SE', format='short') == date(2004, 4, 1)
 
 
+def test_parse_date_custom_format():
+    assert dates.parse_date('1.4.2024', format='dd.mm.yyyy') == date(2024, 4, 1)
+    assert dates.parse_date('2024.4.1', format='yyyy.mm.dd') == date(2024, 4, 1)
+    # Dates that look like ISO 8601 should use the custom format as well:
+    assert dates.parse_date('2024-04-01', format='yyyy.dd.mm') == date(2024, 1, 4)
+
+
 @pytest.mark.parametrize('input, expected', [
     # base case, fully qualified time
     ('15:30:00', time(15, 30)),
@@ -666,6 +686,37 @@ def test_parse_time(input, expected):
     assert dates.parse_time(input, locale='en_US') == expected
 
 
+def test_parse_time_no_seconds_in_format():
+    # parse time using a time format which does not include seconds
+    locale = 'cs_CZ'
+    fmt = 'short'
+    assert dates.get_time_format(format=fmt, locale=locale).pattern == 'H:mm'
+    assert dates.parse_time('9:30', locale=locale, format=fmt) == time(9, 30)
+
+
+def test_parse_time_alternate_characters(monkeypatch):
+    # 'K' can be used as an alternative to 'H'
+    def get_time_format(*args, **kwargs):
+        return parse_pattern('KK:mm:ss')
+    monkeypatch.setattr(dates, 'get_time_format', get_time_format)
+
+    assert dates.parse_time('9:30') == time(9, 30)
+
+
+def test_parse_date_alternate_characters(monkeypatch):
+    # 'l' can be used as an alternative to 'm'
+    def get_date_format(*args, **kwargs):
+        return parse_pattern('yyyy-ll-dd')
+    monkeypatch.setattr(dates, 'get_time_format', get_date_format)
+
+    assert dates.parse_date('2024-10-20') == date(2024, 10, 20)
+
+
+def test_parse_time_custom_format():
+    assert dates.parse_time('15:30:00', format='HH:mm:ss') == time(15, 30)
+    assert dates.parse_time('00:30:15', format='ss:mm:HH') == time(15, 30)
+
+
 @pytest.mark.parametrize('case', ['', 'a', 'aaa'])
 @pytest.mark.parametrize('func', [dates.parse_date, dates.parse_time])
 def test_parse_errors(case, func):
@@ -734,6 +785,365 @@ def test_russian_week_numbering():
     assert dates.format_date(v, format='YYYY-ww', locale='de_DE') == '2016-52'
 
 
+def test_week_numbering_isocalendar():
+    locale = Locale.parse('de_DE')
+    assert locale.first_week_day == 0
+    assert locale.min_week_days == 4
+
+    def week_number(value):
+        return dates.format_date(value, format="YYYY-'W'ww-e", locale=locale)
+
+    for year in range(1991, 2010):
+        for delta in range(-9, 9):
+            value = date(year, 1, 1) + timedelta(days=delta)
+            expected = '%04d-W%02d-%d' % value.isocalendar()
+            assert week_number(value) == expected
+
+def test_week_numbering_monday_mindays_4():
+    locale = Locale.parse('de_DE')
+    assert locale.first_week_day == 0
+    assert locale.min_week_days == 4
+
+    def week_number(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format="YYYY-'W'ww-e", locale=locale)
+
+    def week_of_month(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format='W', locale=locale)
+
+    assert week_number('2003-01-01') == '2003-W01-3'
+    assert week_number('2003-01-06') == '2003-W02-1'
+    assert week_number('2003-12-28') == '2003-W52-7'
+    assert week_number('2003-12-29') == '2004-W01-1'
+    assert week_number('2003-12-31') == '2004-W01-3'
+    assert week_number('2004-01-01') == '2004-W01-4'
+    assert week_number('2004-01-05') == '2004-W02-1'
+    assert week_number('2004-12-26') == '2004-W52-7'
+    assert week_number('2004-12-27') == '2004-W53-1'
+    assert week_number('2004-12-31') == '2004-W53-5'
+    assert week_number('2005-01-01') == '2004-W53-6'
+    assert week_number('2005-01-03') == '2005-W01-1'
+    assert week_number('2005-01-10') == '2005-W02-1'
+    assert week_number('2005-12-31') == '2005-W52-6'
+    assert week_number('2006-01-01') == '2005-W52-7'
+    assert week_number('2006-01-02') == '2006-W01-1'
+    assert week_number('2006-01-09') == '2006-W02-1'
+    assert week_number('2006-12-31') == '2006-W52-7'
+    assert week_number('2007-01-01') == '2007-W01-1'
+    assert week_number('2007-12-30') == '2007-W52-7'
+    assert week_number('2007-12-31') == '2008-W01-1'
+    assert week_number('2008-01-01') == '2008-W01-2'
+    assert week_number('2008-01-07') == '2008-W02-1'
+    assert week_number('2008-12-28') == '2008-W52-7'
+    assert week_number('2008-12-29') == '2009-W01-1'
+    assert week_number('2008-12-31') == '2009-W01-3'
+    assert week_number('2009-01-01') == '2009-W01-4'
+    assert week_number('2009-01-05') == '2009-W02-1'
+    assert week_number('2009-12-27') == '2009-W52-7'
+    assert week_number('2009-12-28') == '2009-W53-1'
+    assert week_number('2009-12-31') == '2009-W53-4'
+    assert week_number('2010-01-01') == '2009-W53-5'
+    assert week_number('2010-01-03') == '2009-W53-7'
+    assert week_number('2010-01-04') == '2010-W01-1'
+
+    assert week_of_month('2003-01-01') == '1'  # Wed
+    assert week_of_month('2003-02-28') == '4'
+    assert week_of_month('2003-05-01') == '1'  # Thu
+    assert week_of_month('2003-08-01') == '0'  # Fri
+    assert week_of_month('2003-12-31') == '5'
+    assert week_of_month('2004-01-01') == '1'  # Thu
+    assert week_of_month('2004-02-29') == '4'
+
+
+def test_week_numbering_monday_mindays_1():
+    locale = Locale.parse('tr_TR')
+    assert locale.first_week_day == 0
+    assert locale.min_week_days == 1
+
+    def week_number(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format="YYYY-'W'ww-e", locale=locale)
+
+    def week_of_month(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format='W', locale=locale)
+
+    assert week_number('2003-01-01') == '2003-W01-3'
+    assert week_number('2003-01-06') == '2003-W02-1'
+    assert week_number('2003-12-28') == '2003-W52-7'
+    assert week_number('2003-12-29') == '2004-W01-1'
+    assert week_number('2003-12-31') == '2004-W01-3'
+    assert week_number('2004-01-01') == '2004-W01-4'
+    assert week_number('2004-01-05') == '2004-W02-1'
+    assert week_number('2004-12-26') == '2004-W52-7'
+    assert week_number('2004-12-27') == '2005-W01-1'
+    assert week_number('2004-12-31') == '2005-W01-5'
+    assert week_number('2005-01-01') == '2005-W01-6'
+    assert week_number('2005-01-03') == '2005-W02-1'
+    assert week_number('2005-12-25') == '2005-W52-7'
+    assert week_number('2005-12-26') == '2006-W01-1'
+    assert week_number('2005-12-31') == '2006-W01-6'
+    assert week_number('2006-01-01') == '2006-W01-7'
+    assert week_number('2006-01-02') == '2006-W02-1'
+    assert week_number('2006-12-31') == '2006-W53-7'
+    assert week_number('2007-01-01') == '2007-W01-1'
+    assert week_number('2007-01-08') == '2007-W02-1'
+    assert week_number('2007-12-30') == '2007-W52-7'
+    assert week_number('2007-12-31') == '2008-W01-1'
+    assert week_number('2008-01-01') == '2008-W01-2'
+    assert week_number('2008-01-07') == '2008-W02-1'
+    assert week_number('2008-12-28') == '2008-W52-7'
+    assert week_number('2008-12-29') == '2009-W01-1'
+    assert week_number('2008-12-31') == '2009-W01-3'
+    assert week_number('2009-01-01') == '2009-W01-4'
+    assert week_number('2009-01-05') == '2009-W02-1'
+    assert week_number('2009-12-27') == '2009-W52-7'
+    assert week_number('2009-12-28') == '2010-W01-1'
+    assert week_number('2009-12-31') == '2010-W01-4'
+    assert week_number('2010-01-01') == '2010-W01-5'
+    assert week_number('2010-01-03') == '2010-W01-7'
+    assert week_number('2010-01-04') == '2010-W02-1'
+
+    assert week_of_month('2003-01-01') == '1'  # Wed
+    assert week_of_month('2003-02-28') == '5'
+    assert week_of_month('2003-06-01') == '1'  # Sun
+    assert week_of_month('2003-12-31') == '5'
+    assert week_of_month('2004-01-01') == '1'  # Thu
+    assert week_of_month('2004-02-29') == '5'
+
+
+def test_week_numbering_sunday_mindays_1():
+    locale = Locale.parse('en_US')
+    assert locale.first_week_day == 6
+    assert locale.min_week_days == 1
+
+    def week_number(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format="YYYY-'W'ww-e", locale=locale)
+
+    def week_of_month(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format='W', locale=locale)
+
+    assert week_number('2003-01-01') == '2003-W01-4'
+    assert week_number('2003-01-05') == '2003-W02-1'
+    assert week_number('2003-12-27') == '2003-W52-7'
+    assert week_number('2003-12-28') == '2004-W01-1'
+    assert week_number('2003-12-31') == '2004-W01-4'
+    assert week_number('2004-01-01') == '2004-W01-5'
+    assert week_number('2004-01-04') == '2004-W02-1'
+    assert week_number('2004-12-25') == '2004-W52-7'
+    assert week_number('2004-12-26') == '2005-W01-1'
+    assert week_number('2004-12-31') == '2005-W01-6'
+    assert week_number('2005-01-01') == '2005-W01-7'
+    assert week_number('2005-01-02') == '2005-W02-1'
+    assert week_number('2005-12-24') == '2005-W52-7'
+    assert week_number('2005-12-25') == '2005-W53-1'
+    assert week_number('2005-12-31') == '2005-W53-7'
+    assert week_number('2006-01-01') == '2006-W01-1'
+    assert week_number('2006-01-08') == '2006-W02-1'
+    assert week_number('2006-12-30') == '2006-W52-7'
+    assert week_number('2006-12-31') == '2007-W01-1'
+    assert week_number('2007-01-01') == '2007-W01-2'
+    assert week_number('2007-01-07') == '2007-W02-1'
+    assert week_number('2007-12-29') == '2007-W52-7'
+    assert week_number('2007-12-30') == '2008-W01-1'
+    assert week_number('2007-12-31') == '2008-W01-2'
+    assert week_number('2008-01-01') == '2008-W01-3'
+    assert week_number('2008-01-06') == '2008-W02-1'
+    assert week_number('2008-12-27') == '2008-W52-7'
+    assert week_number('2008-12-28') == '2009-W01-1'
+    assert week_number('2008-12-31') == '2009-W01-4'
+    assert week_number('2009-01-01') == '2009-W01-5'
+    assert week_number('2009-01-04') == '2009-W02-1'
+    assert week_number('2009-12-26') == '2009-W52-7'
+    assert week_number('2009-12-27') == '2010-W01-1'
+    assert week_number('2009-12-31') == '2010-W01-5'
+    assert week_number('2010-01-01') == '2010-W01-6'
+    assert week_number('2010-01-03') == '2010-W02-1'
+
+    assert week_of_month('2003-01-01') == '1'  # Wed
+    assert week_of_month('2003-02-01') == '1'  # Sat
+    assert week_of_month('2003-02-28') == '5'
+    assert week_of_month('2003-12-31') == '5'
+    assert week_of_month('2004-01-01') == '1'  # Thu
+    assert week_of_month('2004-02-29') == '5'
+
+
+def test_week_numbering_sunday_mindays_4():
+    locale = Locale.parse('pt_PT')
+    assert locale.first_week_day == 6
+    assert locale.min_week_days == 4
+
+    def week_number(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format="YYYY-'W'ww-e", locale=locale)
+
+    def week_of_month(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format='W', locale=locale)
+
+    assert week_number('2003-01-01') == '2003-W01-4'
+    assert week_number('2003-01-05') == '2003-W02-1'
+    assert week_number('2003-12-27') == '2003-W52-7'
+    assert week_number('2003-12-28') == '2003-W53-1'
+    assert week_number('2003-12-31') == '2003-W53-4'
+    assert week_number('2004-01-01') == '2003-W53-5'
+    assert week_number('2004-01-04') == '2004-W01-1'
+    assert week_number('2004-12-25') == '2004-W51-7'
+    assert week_number('2004-12-26') == '2004-W52-1'
+    assert week_number('2004-12-31') == '2004-W52-6'
+    assert week_number('2005-01-01') == '2004-W52-7'
+    assert week_number('2005-01-02') == '2005-W01-1'
+    assert week_number('2005-12-24') == '2005-W51-7'
+    assert week_number('2005-12-25') == '2005-W52-1'
+    assert week_number('2005-12-31') == '2005-W52-7'
+    assert week_number('2006-01-01') == '2006-W01-1'
+    assert week_number('2006-01-08') == '2006-W02-1'
+    assert week_number('2006-12-30') == '2006-W52-7'
+    assert week_number('2006-12-31') == '2007-W01-1'
+    assert week_number('2007-01-01') == '2007-W01-2'
+    assert week_number('2007-01-07') == '2007-W02-1'
+    assert week_number('2007-12-29') == '2007-W52-7'
+    assert week_number('2007-12-30') == '2008-W01-1'
+    assert week_number('2007-12-31') == '2008-W01-2'
+    assert week_number('2008-01-01') == '2008-W01-3'
+    assert week_number('2008-01-06') == '2008-W02-1'
+    assert week_number('2008-12-27') == '2008-W52-7'
+    assert week_number('2008-12-28') == '2008-W53-1'
+    assert week_number('2008-12-31') == '2008-W53-4'
+    assert week_number('2009-01-01') == '2008-W53-5'
+    assert week_number('2009-01-04') == '2009-W01-1'
+    assert week_number('2009-12-26') == '2009-W51-7'
+    assert week_number('2009-12-27') == '2009-W52-1'
+    assert week_number('2009-12-31') == '2009-W52-5'
+    assert week_number('2010-01-01') == '2009-W52-6'
+    assert week_number('2010-01-03') == '2010-W01-1'
+
+    assert week_of_month('2003-01-01') == '1'  # Wed
+    assert week_of_month('2003-02-28') == '4'
+    assert week_of_month('2003-05-01') == '0'  # Thu
+    assert week_of_month('2003-12-31') == '5'
+    assert week_of_month('2004-01-01') == '0'  # Thu
+    assert week_of_month('2004-02-29') == '5'
+
+
+def test_week_numbering_friday_mindays_1():
+    locale = Locale.parse('dv_MV')
+    assert locale.first_week_day == 4
+    assert locale.min_week_days == 1
+
+    def week_number(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format="YYYY-'W'ww-e", locale=locale)
+
+    def week_of_month(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format='W', locale=locale)
+
+    assert week_number('2003-01-01') == '2003-W01-6'
+    assert week_number('2003-01-03') == '2003-W02-1'
+    assert week_number('2003-12-25') == '2003-W52-7'
+    assert week_number('2003-12-26') == '2004-W01-1'
+    assert week_number('2003-12-31') == '2004-W01-6'
+    assert week_number('2004-01-01') == '2004-W01-7'
+    assert week_number('2004-01-02') == '2004-W02-1'
+    assert week_number('2004-12-23') == '2004-W52-7'
+    assert week_number('2004-12-24') == '2004-W53-1'
+    assert week_number('2004-12-30') == '2004-W53-7'
+    assert week_number('2004-12-31') == '2005-W01-1'
+    assert week_number('2005-01-01') == '2005-W01-2'
+    assert week_number('2005-01-07') == '2005-W02-1'
+    assert week_number('2005-12-29') == '2005-W52-7'
+    assert week_number('2005-12-30') == '2006-W01-1'
+    assert week_number('2005-12-31') == '2006-W01-2'
+    assert week_number('2006-01-01') == '2006-W01-3'
+    assert week_number('2006-01-06') == '2006-W02-1'
+    assert week_number('2006-12-28') == '2006-W52-7'
+    assert week_number('2006-12-29') == '2007-W01-1'
+    assert week_number('2006-12-31') == '2007-W01-3'
+    assert week_number('2007-01-01') == '2007-W01-4'
+    assert week_number('2007-01-05') == '2007-W02-1'
+    assert week_number('2007-12-27') == '2007-W52-7'
+    assert week_number('2007-12-28') == '2008-W01-1'
+    assert week_number('2007-12-31') == '2008-W01-4'
+    assert week_number('2008-01-01') == '2008-W01-5'
+    assert week_number('2008-01-04') == '2008-W02-1'
+    assert week_number('2008-12-25') == '2008-W52-7'
+    assert week_number('2008-12-26') == '2009-W01-1'
+    assert week_number('2008-12-31') == '2009-W01-6'
+    assert week_number('2009-01-01') == '2009-W01-7'
+    assert week_number('2009-01-02') == '2009-W02-1'
+    assert week_number('2009-12-24') == '2009-W52-7'
+    assert week_number('2009-12-25') == '2009-W53-1'
+    assert week_number('2009-12-31') == '2009-W53-7'
+    assert week_number('2010-01-01') == '2010-W01-1'
+    assert week_number('2010-01-08') == '2010-W02-1'
+
+    assert week_of_month('2003-01-01') == '1'  # Wed
+    assert week_of_month('2003-02-28') == '5'
+    assert week_of_month('2003-03-01') == '1'  # Sun
+    assert week_of_month('2003-12-31') == '5'
+    assert week_of_month('2004-01-01') == '1'  # Thu
+    assert week_of_month('2004-02-29') == '5'
+
+
+def test_week_numbering_saturday_mindays_1():
+    locale = Locale.parse('fr_DZ')
+    assert locale.first_week_day == 5
+    assert locale.min_week_days == 1
+
+    def week_number(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format="YYYY-'W'ww-e", locale=locale)
+
+    def week_of_month(value):
+        value = date.fromisoformat(value)
+        return dates.format_date(value, format='W', locale=locale)
+
+    assert week_number('2003-01-01') == '2003-W01-5'
+    assert week_number('2003-01-04') == '2003-W02-1'
+    assert week_number('2003-12-26') == '2003-W52-7'
+    assert week_number('2003-12-27') == '2004-W01-1'
+    assert week_number('2003-12-31') == '2004-W01-5'
+    assert week_number('2004-01-01') == '2004-W01-6'
+    assert week_number('2004-01-03') == '2004-W02-1'
+    assert week_number('2004-12-24') == '2004-W52-7'
+    assert week_number('2004-12-31') == '2004-W53-7'
+    assert week_number('2005-01-01') == '2005-W01-1'
+    assert week_number('2005-01-08') == '2005-W02-1'
+    assert week_number('2005-12-30') == '2005-W52-7'
+    assert week_number('2005-12-31') == '2006-W01-1'
+    assert week_number('2006-01-01') == '2006-W01-2'
+    assert week_number('2006-01-07') == '2006-W02-1'
+    assert week_number('2006-12-29') == '2006-W52-7'
+    assert week_number('2006-12-31') == '2007-W01-2'
+    assert week_number('2007-01-01') == '2007-W01-3'
+    assert week_number('2007-01-06') == '2007-W02-1'
+    assert week_number('2007-12-28') == '2007-W52-7'
+    assert week_number('2007-12-31') == '2008-W01-3'
+    assert week_number('2008-01-01') == '2008-W01-4'
+    assert week_number('2008-01-05') == '2008-W02-1'
+    assert week_number('2008-12-26') == '2008-W52-7'
+    assert week_number('2008-12-27') == '2009-W01-1'
+    assert week_number('2008-12-31') == '2009-W01-5'
+    assert week_number('2009-01-01') == '2009-W01-6'
+    assert week_number('2009-01-03') == '2009-W02-1'
+    assert week_number('2009-12-25') == '2009-W52-7'
+    assert week_number('2009-12-26') == '2010-W01-1'
+    assert week_number('2009-12-31') == '2010-W01-6'
+    assert week_number('2010-01-01') == '2010-W01-7'
+    assert week_number('2010-01-02') == '2010-W02-1'
+
+    assert week_of_month('2003-01-01') == '1'  # Wed
+    assert week_of_month('2003-02-28') == '4'
+    assert week_of_month('2003-08-01') == '1'  # Fri
+    assert week_of_month('2003-12-31') == '5'
+    assert week_of_month('2004-01-01') == '1'  # Thu
+    assert week_of_month('2004-02-29') == '5'
+
+
 def test_en_gb_first_weekday():
     assert Locale.parse('en').first_week_day == 0  # Monday in general
     assert Locale.parse('en_US').first_week_day == 6  # Sunday in the US
@@ -742,3 +1152,38 @@ def test_en_gb_first_weekday():
 
 def test_issue_798():
     assert dates.format_timedelta(timedelta(), format='narrow', locale='es_US') == '0s'
+
+
+def test_issue_892():
+    assert dates.format_timedelta(timedelta(seconds=1), format='narrow', locale='pt_BR') == '1 s'
+    assert dates.format_timedelta(timedelta(minutes=1), format='narrow', locale='pt_BR') == '1 min'
+    assert dates.format_timedelta(timedelta(hours=1), format='narrow', locale='pt_BR') == '1 h'
+    assert dates.format_timedelta(timedelta(days=1), format='narrow', locale='pt_BR') == '1 dia'
+    assert dates.format_timedelta(timedelta(days=30), format='narrow', locale='pt_BR') == '1 mês'
+    assert dates.format_timedelta(timedelta(days=365), format='narrow', locale='pt_BR') == '1 ano'
+
+
+def test_issue_1089():
+    assert dates.format_datetime(datetime.now(), locale="ja_JP@mod")
+    assert dates.format_datetime(datetime.now(), locale=Locale.parse("ja_JP@mod"))
+
+
+@pytest.mark.parametrize(('locale', 'format', 'negative', 'expected'), [
+    ('en_US', 'long', False, 'in 3 hours'),
+    ('en_US', 'long', True, '3 hours ago'),
+    ('en_US', 'narrow', False, 'in 3h'),
+    ('en_US', 'narrow', True, '3h ago'),
+    ('en_US', 'short', False, 'in 3 hr.'),
+    ('en_US', 'short', True, '3 hr. ago'),
+    ('fi_FI', 'long', False, '3 tunnin päästä'),
+    ('fi_FI', 'long', True, '3 tuntia sitten'),
+    ('fi_FI', 'short', False, '3 t päästä'),
+    ('fi_FI', 'short', True, '3 t sitten'),
+    ('sv_SE', 'long', False, 'om 3 timmar'),
+    ('sv_SE', 'long', True, 'för 3 timmar sedan'),
+    ('sv_SE', 'short', False, 'om 3 tim'),
+    ('sv_SE', 'short', True, 'för 3 tim sedan'),
+])
+def test_issue_1162(locale, format, negative, expected):
+    delta = timedelta(seconds=10800) * (-1 if negative else +1)
+    assert dates.format_timedelta(delta, add_direction=True, format=format, locale=locale) == expected
